Index: main/pbx.c
===================================================================
--- main/pbx.c	(revision 97753)
+++ main/pbx.c	(working copy)
@@ -1905,7 +1905,7 @@
 /*! \brief  ast_extensions_state2: Check state of extension by using hints */
 static int ast_extension_state2(struct ast_exten *e)
 {
-	char hint[AST_MAX_EXTENSION];
+	char hint[AST_MAX_EXTENSION] = "";
 	char *cur, *rest;
 	int allunavailable = 1, allbusy = 1, allfree = 1, allonhold = 1;
 	int busy = 0, inuse = 0, ring = 0;
@@ -2099,6 +2099,20 @@
 		return -1;
 	}
 
+	/* If this is a pattern, dynamically create a new extension for this
+	 * particular match.  Note that this will only happen once for each
+	 * individual extension, because the pattern will no longer match first.
+	 */
+	if (e->exten[0] == '_') {
+		ast_add_extension(e->parent->name, 0, exten, e->priority, e->label,
+			e->cidmatch, e->app, strdup(e->data), free,
+			e->registrar);
+		e = ast_hint_extension(NULL, context, exten);
+		if (!e || e->exten[0] == '_') {
+			return -1;
+		}
+	}
+
 	/* Find the hint in the list of hints */
 	AST_LIST_LOCK(&hints);
 
@@ -3996,6 +4010,16 @@
 	while ((this = AST_LIST_REMOVE_HEAD(&store, list))) {
 		struct pbx_find_info q = { .stacklen = 0 };
 		exten = pbx_find_extension(NULL, NULL, &q, this->context, this->exten, PRIORITY_HINT, NULL, "", E_MATCH);
+		/* If this is a pattern, dynamically create a new extension for this
+		 * particular match.  Note that this will only happen once for each
+		 * individual extension, because the pattern will no longer match first.
+		 */
+		if (exten && exten->exten[0] == '_') {
+			ast_add_extension(exten->parent->name, 0, this->exten, PRIORITY_HINT, NULL,
+				0, exten->app, strdup(exten->data), free, registrar);
+			exten = ast_hint_extension(NULL, this->context, this->exten);
+		}
+
 		/* Find the hint in the list of hints */
 		AST_LIST_TRAVERSE(&hints, hint, list) {
 			if (hint->exten == exten)
@@ -4780,7 +4804,18 @@
 	   contains variable references, then expand them
 	*/
 	ast_mutex_lock(&globalslock);
-	if (priority == PRIORITY_HINT && AST_LIST_FIRST(&globals) && strstr(application, "${")) {
+	if (priority == PRIORITY_HINT && AST_LIST_FIRST(&globals) && strstr(application, "${") && !strstr(extension, "_")) {
+		struct ast_channel c = {0, };
+		/* 	Moved the extension parsing to occur only when we are adding a 
+			non "_" extension and evaluate the EXTEN parameter here.  This now 
+			supports the Custom: context 
+		*/
+
+		ast_copy_string(c.exten, extension, sizeof(c.exten));
+		ast_copy_string(c.context, con->name, sizeof(c.context));
+		pbx_substitute_variables_helper(&c, application, expand_buf, sizeof(expand_buf));
+		application = expand_buf;
+
 		pbx_substitute_variables_varshead(&globals, application, expand_buf, sizeof(expand_buf));
 		application = expand_buf;
 	}
